/*
 * ***********************************************************************
 *
 * File 'CanvasPlusGeomTools.h'.
 *
 * CanvasPlus drawing rotines interface file.
 *
 * Written by Jorge Lima in Nov 23 14:44:03 WET 2004
 *
 */

#ifndef _CP_GEOM_TOOLS_H
#define _CP_GEOM_TOOLS_H

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Physical point
 */
typedef struct CP_Point {
  double x, y;
} CP_Point;

/*
 * Physical area
 */
typedef struct CP_Area {
  double x1, y1, x2, y2;
} CP_Area;

#define CP_AREA_DISJOINT     0
#define CP_AREA_INTERSECT    1
#define CP_AREA_CONTAINS     2
#define CP_AREA_ISCONTAINED  3

inline int compareAreas(const CP_Area *a1, const CP_Area *a2)
{
  if (a1->x1>a2->x2 || a1->x2<a2->x1 || a1->y1>a2->y2 || a1->y2<a2->y1)
    return CP_AREA_DISJOINT;
  else if (a1->x1>a2->x1 && a1->x2<a2->x2 && a1->y1>a2->y1 && a1->y2<a2->y2)
    return CP_AREA_ISCONTAINED;
  else if (a1->x1<a2->x1 && a1->x2>a2->x2 && a1->y1<a2->y1 && a1->y2>a2->y2)
    return CP_AREA_CONTAINS;
  else 
    return CP_AREA_INTERSECT;
}

inline int pointInArea(const CP_Area *a, const CP_Point *p)
{
  if (a->x1>p->x || a->x2<p->x || a->y1>p->y || a->y2<p->y)
    return 0;
  else 
    return 1;
}

inline CP_Area surroundingArea(const CP_Point *pt, double dist)
{
  CP_Area area;
  area.x1 = pt->x - dist; area.x2 = pt->x + dist;
  area.y1 = pt->y - dist; area.y2 = pt->y + dist;
  return area;
} 

#ifdef __cplusplus
}
#endif

#endif /* _CP_GEOM_TOOLS_H */
