package require Mtiwidgets
package require Iwidgets
package require callib


proc ToDoUpdate {callarg} {
    global ToDoArray currentToDoList currentDate todo calendarLabel

    foreach {year month day dayname column row} $callarg {}

    set date [format "%02d/%02d/%04d" $month $day $year]
    if {[info exists currentDate] &&
	$currentDate ne "" &&
	[info exists currentToDoList] &&
	$currentToDoList ne ""} {
	set ToDoArray($currentDate) $currentToDoList
    }
    $todo selection clear 0 end
    if {[info exists ToDoArray($date)]} {
	set currentToDoList $ToDoArray($date)
    } else {
	set currentToDoList [list]
    }
    set currentDate $date
    set calendarLabel [clock format [clock scan $date] -format "%B %Y"]
}

proc NotesUpdate {args} {
	global currentDate currentToDo NotesArray notes todo

	if {[info exists currentToDo] &&
	    $currentToDo ne ""} {
		set NotesArray($currentToDo) [$notes get 1.0 end-1c]
		$notes delete 1.0 end
		unset currentToDo
	}
	set cur [$todo curselection]
	if {$cur ne ""} {
		set currentToDo [$todo get [$todo curselection]]
		if {[info exists NotesArray($currentToDo)]} {
			$notes insert end $NotesArray($currentToDo)
		}
	}
}

set ToDoArray(09/15/2005) [list "Finish Paper"]
set ToDoArray(09/16/2005) [list "ModelSim University"]
set NotesArray([lindex $ToDoArray(09/16/2005) 0]) "All material due to Ross for publication.\n"

set dbpath [file join $env(HOME) .mtical]

proc LoadDB {dbpath} {
    global ToDoArray NotesArray
    set path ${dbpath}.todo
    if {[file exists $path]} {
	set fp [open $path]
	set l1 [read $fp]
	if {([llength $l1] & 1) == 0} {
	    array set ToDoArray $l1
	} else {
	    puts stderr "Bad data in $path"
	}
	close $fp
    }
    set path ${dbpath}.notes
    if {[file exists $path]} {
	set fp [open $path]
	set l2 [read $fp]
	if {([llength $l2] & 1) == 0} {
	    array set NotesArray $l2
	} else {
	    puts stderr "Bad data in $path"
	}
	close $fp
    }
}

proc SaveDB {dbpath} {
    global ToDoArray NotesArray
    set path ${dbpath}.todo
    set fp [open $path w]
    puts $fp [array get ToDoArray]
    close $fp
    set path ${dbpath}.notes
    set fp [open $path w]
    puts $fp [array get NotesArray]
}

trace add variable currentDate write NotesUpdate


mtiwidgets::panemanager .mp -showhandle 0 -sashwidth 6 -sashpad 0 -sashrelief flat -borderwidth 0
mtiwidgets::paneframe .mp.notes -text "Notes" -activebackground {steel blue}
mtiwidgets::paneframe .mp.todo  -text "To Do" -activebackground {steel blue}
mtiwidgets::paneframe .mp.msgs  -text "Messages" -activebackground {steel blue}
mtiwidgets::paneframe .mp.cal   -text "Calendar" -activebackground {steel blue}
mtiwidgets::paneframe .mp.clk   -text "Clock" -activebackground {steel blue}

set cs1 [.mp.notes childsite]
set cs2 [.mp.todo childsite]
set cs3 [.mp.msgs childsite]
set cs4 [.mp.cal childsite]
set cs5 [.mp.clk childsite]

set notes [text $cs1.notes -bg white -width 1 -height 1 -wrap word]
set todo [listbox $cs2.todo -listvariable currentToDoList -selectmode single]
listbox $cs3.msgs -listvariable msgList

set commonOpt "-relief flat -bd 0 -background white -highlightthickness 0"
set callbl [label $cs4.callbl -textvariable calendarLabel {expand}$commonOpt]
set cal [calwid $cs4.cal -callback ToDoUpdate \
	     -daynames {Sun Mon Tue Wed Thu Fri Sat} \
	     -background white \
	     -activebackground {light gray}]

set calprv [button $cs4.prv -text "<" {expand}$commonOpt -command [list PrevMonth $cal]]
set calnxt [button $cs4.nxt -text ">" {expand}$commonOpt -command [list NextMonth $cal]]
set calpry [button $cs4.pry -text "<<" {expand}$commonOpt -command [list $cal prevyear]]
set calnxy [button $cs4.nxy -text ">>" {expand}$commonOpt -command [list $cal nextyear]]

set clock [iwidgets::watch $cs5.clock \
	       -clockcolor white \
	       -hourcolor white \
	       -minutecolor white \
	       -secondcolor blue \
	       -pivotcolor white \
	       -tickcolor black]

bind $todo <<ListboxSelect>> NotesUpdate

.mp.notes configure -window $cs1.notes \
    -expelcommand ".mp togglerip .mp.notes" -maximizecommand ".mp togglemax .mp.notes"
.mp.todo configure -window $cs2.todo \
    -expelcommand ".mp togglerip .mp.todo" -maximizecommand ".mp togglemax .mp.todo"
.mp.msgs configure -window $cs3.msgs \
    -expelcommand ".mp togglerip .mp.msgs" -maximizecommand ".mp togglemax .mp.msgs"
.mp.cal configure -window $cs4.cal \
    -expelcommand ".mp togglerip .mp.cal" -maximizecommand ".mp togglemax .mp.cal"
.mp.clk configure -window $cs5.clock \
    -expelcommand ".mp togglerip .mp.clk" -maximizecommand ".mp togglemax .mp.clk"

pack $cs1.notes -fill both -expand 1
pack $cs2.todo -fill both -expand 1
pack $cs3.msgs -fill both -expand 1

grid $calpry $calprv $callbl $calnxt $calnxy -sticky ewns
grid $cs4.cal -       -           -       -      -sticky nsew
grid columnconfigure $cs4 2 -weight 1
grid rowconfigure $cs4 1 -weight 1

pack $cs5.clock -fill both -expand true

.mp add .mp.todo {} w -stretch never
.mp add .mp.notes .mp.todo e -stretch always
.mp add .mp.msgs .mp.todo s -stretch never
.mp add .mp.cal .mp.msgs e
.mp add .mp.clk .mp.cal e

pack .mp -fill both -expand 1

proc NextMonth {cal} {
    set mn [$cal configure -month]
    set yr [$cal configure -year]
    
    if {[incr mn] == 13} {
	set mn 1
	incr yr
    }
    $cal configure -month $mn -year $yr
    ToDoUpdate [list $yr $mn 1 {} {} {}]
}
proc PrevMonth {cal} {
    set mn [$cal configure -month]
    set yr [$cal configure -year]
    
    if {[incr mn -1] == 0} {
	set mn 12
	incr yr -1
    }
    $cal configure -month $mn -year $yr
    ToDoUpdate [list $yr $mn 1 {} {} {}]
}

proc ClockUpdate {} {
	global clock
	$clock show now
	after 1000 ClockUpdate
}

ClockUpdate

set layout {horizontal 
    {
	{
	    .mp.todo {-height 204 -minsize 40 -stretch never -width 712} 204 na 
	    {
		.mp.msgs {-height 265 -minsize 40 -stretch never -width 152} 152 na 
		.mp.cal {-height 265 -minsize 40 -width 324} 482 na 
		.mp.clk {-height 265 -minsize 40 -width 224} 712 na
	    } {-height 265 -minsize 40 -stretch always -width 712} 475 na
	} {-height 475 -minsize 40 -stretch always -width 712} 495 na 
	.mp.notes {-height 475 -minsize 40 -stretch always -width 124} 842 na
    }
}


LoadDB $dbpath

proc LoadCal {} {
    global ToDoArray
    foreach date [array names ToDoArray] {
	foreach {month day year} [split $date /] {}
	foreach todo $ToDoArray($date) {
	    set mark [list [string trim $day 0] [string trim $month 0] [string trim $year 0] 1 blue $todo]
	    $::cal configure -mark $mark
	}
    }
}

LoadCal

ToDoUpdate [clock format [clock seconds] -format "%Y %m %d %A {} {}"]



wm geometry . 842x475+43+22
update
.mp unserialize $layout

bind . <Destroy> "SaveDB $dbpath"
